
#include <Servo.h>

/*--Define--*/
/*Propulseur*/
#define Propulse_D 9
#define Propulse_G 5
/*Ballaste*/
#define PWM_Ballast_G 3
#define in1_Ballast_G 2
#define in2_Ballast_G 4
#define PWM_Ballast_D 6
#define in1_Ballast_D 7
#define in2_Ballast_D 8

/*--Variable--*/
/*Propulseur*/
Servo Propulseur_D, Propulseur_G;
int tempsPDF = 0;
int tempsPGF = 0;
long tempsPD = 0;
long tempsPG = 0;

/*Ballaste*/
int tempsBDF = 0;
int tempsBGF = 0;
long tempsBD = 0;
long tempsBG = 0;
/*Protocol commande moteur*/
bool flagCmd = 0;
String cmd;

void setup() {
  Serial.begin(57600);
  /*Initialisation des propulseurs*/
  Propulseur_D.attach(Propulse_D, 1000, 2000) ;
  Propulseur_G.attach(Propulse_G, 1000, 2000) ;
  Propulseur_D.write(0); // vitesse mise à 0 moteur droit
  Propulseur_G.write(0); // vitesse mise à 0 moteur gauche
  /*Initialisation des ballastes*/
  pinMode(PWM_Ballast_G, OUTPUT);
  pinMode(in1_Ballast_G, OUTPUT);
  pinMode(in2_Ballast_G, OUTPUT);
  pinMode(PWM_Ballast_D, OUTPUT);
  pinMode(in1_Ballast_D, OUTPUT);
  pinMode(in2_Ballast_D, OUTPUT);

  Ballast_Fonction(255, '1', 255, '1');
  delay(2000);
  Ballast_Fonction(0, '0', 0, '0');
  Serial.println('Entrer une commande pour tester les propulseurs ou ballasts');
  Serial.print('Exemple propulseurs : P18010018010000');
  Serial.print('Exemple ballasts : B25505025505022');
}
void loop() {
  if(Serial.available()){
  cmd = Serial.read();
  flagCmd = 1;
  }

  /*Decodage si réception d'une commande*/
  if (flagCmd == 1) {
    decodeCommand();
    flagCmd = 0;
  }
  
  /*Check des délais moteurs de la commande*/
  checkMotorDelay();
}
/*Fonction de contrôle des ballates*/
/*vitesse : 0 à 255 | moteur Droit,Gauche = 0,1 | rotation 0,1,2 = arrêt,horaire,a-hor|*/
void Ballast_Fonction(int vitesseG, char rotG , int vitesseD, char rotD) {
  analogWrite(PWM_Ballast_G, vitesseG);
  tempsBG = millis();
  switch (rotG) {
    case '0':
      digitalWrite(in1_Ballast_G, LOW);
      digitalWrite(in2_Ballast_G, LOW);
      break;

    case '1':
      digitalWrite(in1_Ballast_G, HIGH);
      digitalWrite(in2_Ballast_G, LOW);
      break;

    case '2':
      digitalWrite(in1_Ballast_G, LOW);
      digitalWrite(in2_Ballast_G, HIGH);
      break;

    default:
      break;
  }
  analogWrite(PWM_Ballast_D, vitesseD);
  tempsBD = millis();
  switch (rotD) {
    case '0':
      digitalWrite(in1_Ballast_D, LOW);
      digitalWrite(in2_Ballast_D, LOW);
      break;

    case '1':
      digitalWrite(in1_Ballast_D, HIGH);
      digitalWrite(in2_Ballast_D, LOW);
      break;

    case '2':
      digitalWrite(in1_Ballast_D, LOW);
      digitalWrite(in2_Ballast_D, HIGH);
      break;

    default:
      break;
  }
}
/*Fonction de decodage du protocol de commande*/
void decodeCommand() {
  int vitesseD;
  int vitesseG;
  vitesseD = cmd.substring(1, 4).toInt();
  vitesseG = cmd.substring(7, 10).toInt();
  if (cmd[0] == 'P') {
    tempsPDF = cmd.substring(4, 7).toInt();
    tempsPGF = cmd.substring(10, 13).toInt();
    propulseur_Fonction(vitesseD, vitesseG);
  }
  else if (cmd[0] == 'B') {
    tempsBDF = cmd.substring(4, 7).toInt();
    tempsBGF = cmd.substring(10, 13).toInt();
    char rotationD = cmd[13];
    char rotationG = cmd[14];
    Ballast_Fonction(vitesseG, rotationG, vitesseD, rotationD);
  }
}
/*Fonction de contrôle des ballates*/
/*vitesse moteur : 0 à 180*/
void propulseur_Fonction(int vitesseD, int vitesseG) {
  tempsPD = millis();
  Propulseur_D.write(vitesseD);
  tempsPG = millis();
  Propulseur_G.write(vitesseG);
}
void checkMotorDelay()
{
  /*Potentiel problème d'overfloat de millis et non passage dans la condition*/
  if ((millis() - tempsPD) > tempsPDF * 100)
  {
    Propulseur_D.write(0);
  }
  if ((millis() - tempsPG) > tempsPGF * 100)
  {
    Propulseur_G.write(0);
  }
  if ((millis() - tempsBD) > tempsBDF * 100)
  {
    analogWrite(PWM_Ballast_D, 0);
  }
  if ((millis() - tempsBG) > tempsBGF * 100)
  {
    analogWrite(PWM_Ballast_G, 0);
  }
}
